@@ -1513,7 +1513,7 @@ void Logger::CodeDisableOptEvent(Handle<AbstractCode> code,
   MSG_BUILDER();
   msg << kLogEventsNames[CodeEventListener::CODE_DISABLE_OPT_EVENT] << kNext
       << shared->DebugNameCStr().get() << kNext
-      << GetBailoutReason(shared->disable_optimization_reason());
+      << GetBailoutReason(shared->disabled_optimization_reason());
   msg.WriteToLogFile();
 }
 
@@ -1976,7 +1976,6 @@ void Logger::LogAccessorCallbacks() {
 }
 
 void Logger::LogAllMaps() {
-  DisallowGarbageCollection no_gc;
   Heap* heap = isolate_->heap();
   CombinedHeapObjectIterator iterator(heap);
   for (HeapObject obj = iterator.Next(); !obj.is_null();
@@ -2245,13 +2244,14 @@ void ExistingCodeLogger::LogCompiledFunctions() {
       LogExistingFunction(
           shared,
           Handle<AbstractCode>(
-              AbstractCode::cast(shared->InterpreterTrampoline()), isolate_));
+              AbstractCode::cast(FromCodeT(shared->InterpreterTrampoline())),
+              isolate_));
     }
     if (shared->HasBaselineCode()) {
-      LogExistingFunction(
-          shared, Handle<AbstractCode>(
-                      AbstractCode::cast(shared->baseline_code(kAcquireLoad)),
-                      isolate_));
+      LogExistingFunction(shared, Handle<AbstractCode>(
+                                      AbstractCode::cast(FromCodeT(
+                                          shared->baseline_code(kAcquireLoad))),
+                                      isolate_));
     }
     if (pair.second.is_identical_to(BUILTIN_CODE(isolate_, CompileLazy)))
       continue;
