@@ -314,9 +314,10 @@ struct V8_EXPORT_PRIVATE WasmModule {
   std::vector<TypeDefinition> types;  // by type index
   std::vector<uint8_t> type_kinds;    // by type index
   std::vector<uint32_t> supertypes;   // by type index
-  // Map from each type index to the index of its corresponding canonical type.
+  // Map from each type index to the index of its corresponding canonical index.
+  // Canonical indices do not correspond to types.
   // Note: right now, only functions are canonicalized, and arrays and structs
-  // map to themselves.
+  // map to 0.
   std::vector<uint32_t> canonicalized_type_ids;
 
   bool has_type(uint32_t index) const { return index < types.size(); }
@@ -462,7 +463,8 @@ int GetNearestWasmFunction(const WasmModule* module, uint32_t byte_offset);
 // Returns 0 if the type has no explicit supertype.
 // The result is capped to {kV8MaxRttSubtypingDepth + 1}.
 // Invalid cyclic hierarchies will return -1.
-int GetSubtypingDepth(const WasmModule* module, uint32_t type_index);
+V8_EXPORT_PRIVATE int GetSubtypingDepth(const WasmModule* module,
+                                        uint32_t type_index);
 
 // Interface to the storage (wire bytes) of a wasm module.
 // It is illegal for anyone receiving a ModuleWireBytes to store pointers based
